#'---
#' author: Gabriel E. Cabrera
#' title: Comparing Out-Of-Sample Forecasts
#' date: Last updated `r Sys.Date()`
#'---

# packages
library("dplyr")
library("readxl")

start_time <- Sys.time()

# auxiliary functions
source("Functions/clark-west.R")
source("Functions/regress.R")

# read rapach and zhou (2013) dataset
xs <- read_xlsx("Dataset/PredictorData2010.xlsx", sheet = 1)

# Out-Of-Sample Forecasting -----------------------------------------------

n_row <- nrow(xs) - 1
n_col <- ncol(xs) - 2

r <- (1957 - 1927) * 12
p <- n_row - r

fc_econ <- matrix(NA, nrow = p, ncol = n_col)
colnames(fc_econ) <- colnames(xs[,3:ncol(xs)])

fc_ha <- matrix(NA, nrow = p, ncol = 1)
colnames(fc_ha) <- c("ha")

actual <- matrix(NA, nrow = p, ncol = 1)
colnames(actual) <- c("actual")

for (i in 1:p) {
  
  cat("Iteration:", i, "of", p, "\n")
  
  for (j in 1:n_col) {
    
    # get target + predictor
    y_i <- xs[2:(r + i), 2] %>% as.matrix()
    x_i_j <- xs[2:(r + i), (2 + j)] %>% as.matrix()
    
    # create ones vector
    ones <- rep(1, length(x_i_j))

    # regression
    res <- regress(y_i, cbind(ones, x_i_j))[[1]]

    # prediction out-of-sample
    y_hat_i_j <- res[1] + res[2] * xs[(r + i + 1),(2 + j)]
    
    # save prediction
    fc_econ[i, j] <- as.numeric(y_hat_i_j)

  }
  
  # historical average
  fc_ha[i, 1] <- xs[1:(r + i), 2] %>%
    summarise(ha = mean(eqp)) %>% 
    pull(1)
  
  # actual
  actual[i, 1] <- xs[(r + i + 1), 2] %>% pull()
  
}

# Compare Out-Of-Sample Forecast ------------------------------------------

oosres <- matrix(NA, nrow = n_col, ncol = 2)

colnames(oosres) <- c("r2os", "p-value")
rownames(oosres) <- colnames(xs[,3:ncol(xs)])

for (k in 1:n_col) {
  
  cw_res <- clark_west(actual, fc_ha, fc_econ[,k])
  
  oosres[k, 1] <- cw_res[[2]]
  oosres[k, 2] <- cw_res[[4]]
  
}

# Replicate Table 6.1 > Panel A: Unrestricted predictive regression forecast
round(oosres, 2)

end_time <- Sys.time()

elasped_time <- end_time - start_time

cat('\n')
cat(sprintf('%s%#.2f\n', 'elapsed time = ', elasped_time))
